/* ----------------------------------------------------------------------- */

#include <stdbool.h>
#include "stm32f10x_lib.h"
#include "system.h"
#include "n6610_lcd.h"

/* ----------------------------------------------------------------------- */
// Philips PCF8833 LCD controller command codes
enum pcf8833_cmd 
{
 	NOP = 0x00, // nop
	SWRESET = 0x01, // software reset
	BSTROFF =  0x02, // booster voltage OFF
	BSTRON  = 0x03, // booster voltage ON
	RDDIDIF =  0x04, // read display identification
	R0DDST =  0x09, // read display status
	SLEEPOUT =  0x11, // sleep out
	PTLON =  0x12, // partial display mode
	NORON =  0x13, // display normal mode	
	INVOFF =  0x20, // inversion OFF
	INVON =  0x21, // inversion ON
	DALO =  0x22, // all pixel OFF
	DAL =  0x23, // all pixel ON
	SETCON =  0x25, // write contrast
	DISPOFF =  0x28, // display OFF
	DISPON =  0x29, // display ON
	CASET =  0x2A, // column address set
	PASET =  0x2B, // page address set
	RAMWR =  0x2C, // memory write
	RGBSET =  0x2D, // colour set
	PTLAR =  0x30, // partial area
	VSCRDEF =  0x33, // vertical scrolling definition
	TEOFF =  0x34, // test mode
	TEON =  0x35, // test mode
	MADCTL =  0x36, // memory access control
	SEP =  0x37, // vertical scrolling start address
	IDMOFF =  0x38, // idle mode OFF
	IDMON =  0x39, // idle mode ON
	COLMOD =  0x3A, // interface pixel format
	SETVOP =  0xB0, // set Vop
	BRS =  0xB4, // bottom row swap
	TRS =  0xB6, // top row swap
	DISCTR =  0xB9, // display control
	DOR =  0xBA, // data order
	DATCTL =  0xBC, // Data scan direction, etc.
	TCDFE =  0xBD, // enable/disable DF temperature compensation
	TCVOPE =  0xBF, // enable/disable Vop temp comp
	EC =  0xC0, // internal or external oscillator
	SETMUL =  0xC2, // set multiplication factor
	TCVOPAB =  0xC3, // set TCVOP slopes A and B
	TCVOPCD =  0xC4, // set TCVOP slopes c and d
	TCDF =  0xC5, // set divider frequency
	DF8COLOR =  0xC6, // set divider frequency 8-color mode
	SETBS =  0xC7, // set bias system
	RDTEMP =  0xC8, // temperature read back
	NLI =  0xC9, // n-line inversion
	RDID1 =  0xDA, // read ID1
	RDID2 =  0xDB, // read ID2
	RDID3 =  0xDC // read ID3
};

/* ----------------------------------------------------------------------- */
//SPI port
#define SPI_PORT GPIOA
#define SPI_APB2ENR_PORT RCC_APB2Periph_GPIOA
//SPI pin bits
#define MOSI_BIT 7
#define SCK_BIT 5
//Display control bits
#define CTRL_PORT GPIOD
#define CTRL_APB2ENR_PORT RCC_APB2Periph_GPIOD
#define SCE_BIT 2
#define RES_BIT 1

/* ----------------------------------------------------------------------- */
// I/O port lines control functions
static inline void res(bool en)
{
    if(en) io_set(CTRL_PORT, RES_BIT);
    else io_clr(CTRL_PORT, RES_BIT);
}

/* ----------------------------------------------------------------------- */
static inline void sce(bool en)
{
    if(en) io_set(CTRL_PORT, SCE_BIT);
    else io_clr(CTRL_PORT, SCE_BIT);
}
/* ----------------------------------------------------------------------- */
static inline void sck(bool en)
{
    if(en) io_set(SPI_PORT, SCK_BIT );
    else io_clr(SPI_PORT, SCK_BIT );
}
/* ----------------------------------------------------------------------- */
static inline void mosi(bool en)
{
    if(en) io_set(SPI_PORT, MOSI_BIT );
    else io_clr(SPI_PORT, MOSI_BIT );
}

/* ----------------------------------------------------------------------- */
// SPI functions
static void transfer_spi(unsigned char data)
{
    unsigned char i;
    sck(1);
    for (i = 0; i < 8; i++) 
    {
      
      sck(0);
      nop(); nop();
      mosi(data&0x80);
      nop(); nop();
      sck(1);
      nop(); nop();
      data <<= 1;
    }
    sce(1);
}
/* ----------------------------------------------------------------------- */
static void transfer_spi_command(unsigned char cmd)
{
	sce(0);
	sck(0);
	mosi(0); // For commands the first bit is 0
	transfer_spi(cmd);
}

/* ----------------------------------------------------------------------- */
static void transfer_spi_data(unsigned char data)
{
	sce(0);
	sck(0);
	mosi(1); 	// For data the first bit is 1
	transfer_spi(data);
}


/* ----------------------------------------------------------------------- */
// This function sets contrast. con should be value in range <-64...63>
void SetContrast(char con)
{
	transfer_spi_command(SETCON);
	transfer_spi_data(con);
}
/* ----------------------------------------------------------------------- */
// Initialize display
void nlcd_init(void) 
{
	int i;
	// Hardware reset
	RCC->APB2ENR |=  SPI_APB2ENR_PORT | CTRL_APB2ENR_PORT;
	io_config(CTRL_PORT,SCE_BIT,GPIO_MODE_50MHZ,GPIO_CNF_GPIO_PP);
	io_config(CTRL_PORT,RES_BIT,GPIO_MODE_50MHZ,GPIO_CNF_GPIO_PP);
	io_config(SPI_PORT,MOSI_BIT,GPIO_MODE_50MHZ,GPIO_CNF_GPIO_PP);
	io_config(SPI_PORT,SCK_BIT,GPIO_MODE_50MHZ,GPIO_CNF_GPIO_PP);
	sce(1);
	sck(1);
	mosi(1);
	res(0);
	for(i = 0; i < 80000; i++) nop();
	res(1);
	for(i = 0; i < 80000; i++) nop();

	// Sleep out
	transfer_spi_command(SLEEPOUT);

	// Inversion on (command 0x20)
	transfer_spi_command(INVOFF); 

	// Color Interface Pixel Format (command 0x3A)
	transfer_spi_command(COLMOD);
	transfer_spi_data(0x03); // 0x03 = 12 bits-per-pixel

	// Memory access controler (command 0x36)
	transfer_spi_command(MADCTL);
	transfer_spi_data(0x50);

	for(i = 0; i < 2000; i++) nop();

	// Display On (command 0x29)
	transfer_spi_command(DISPON);	
}

/* ----------------------------------------------------------------------- */
// Set all pixels to color
void nlcd_clear_screen(void) 
{
	int i;
	// Set row address range
	transfer_spi_command(PASET);
	transfer_spi_data(0);
	transfer_spi_data(131);

	// Set column address range
	transfer_spi_command(CASET);
	transfer_spi_data(0);
	transfer_spi_data(131);

	// Write memory
	transfer_spi_command(RAMWR);
	for(i = 0; i < (131 * 131); i++) 
	{
	   transfer_spi_data(0);
	   transfer_spi_data(0);
	}
}

/* ----------------------------------------------------------------------- */
// Set x, y pixel to color
void nlcd_set_pixel(int x, int y, unsigned int color)
{
	// Set columns address range
	transfer_spi_command(CASET); 
	transfer_spi_data(x);
	transfer_spi_data(131);
	
	// Set rows address range
	transfer_spi_command(PASET);
	transfer_spi_data(y);
	transfer_spi_data(131);

	// Write memory
	transfer_spi_command(RAMWR);
	transfer_spi_data(color);
	transfer_spi_data(color >> 4);
}
/* ----------------------------------------------------------------------- */
